import logging
import os

import bbcloud_python_sdk as utils
from .ObsFileCache import ObsFileCache

from .OssFileCache import OssFileCache

from .SynologyFileCache import SynologyFileCache


def runtime_logging(func):
    def wrap(*args, **kwargs):
        res = func(*args, **kwargs)
        logging.info(
            'FileCache:%s:%s:%s:%s' % (func.__name__, args[0].namespace, args if len(args) > 1 else kwargs, res))
        return res

    return wrap


class FileCache():
    def __init__(self, available_area, using_area):
        self.namespace = ''
        self.using_area = using_area
        self.available_area = available_area
        self.default_engine = self.getEngine(using_area=using_area)

    def getEngine(self, using_area=None):
        if not using_area:
            using_area = self.using_area
        engine = self.available_area[using_area]['engine']
        config = self.available_area[using_area]['config']
        if engine == 'OssFileCache':
            return OssFileCache(access_key_id=config['access_key_id'],
                                access_key_secret=config['access_key_secret'],
                                endpoint=config['endpoint'],
                                bucket_name=config['bucket_name'],
                                cache_path_root=config.get('cache_path_root', 'OssFileCache')
                                )
        elif engine == 'SynologyFileCache':
            return SynologyFileCache(
                ip_address=config['ip_address'],
                port=config['port'],
                username=config['username'],
                password=config['password'],
                cache_path_root=config.get('cache_path_root', 'SynologyFileCache')
            )
        elif engine == 'ObsFileCache':
            return ObsFileCache(
                access_key_id=config['access_key_id'],
                secret_access_key=config['access_key_secret'],
                endpoint=config['endpoint'],
                bucket_name=config['bucket_name'],
                cache_path_root=config.get('cache_path_root', 'ObsFileCache'))

    def set_namespace(self, namespace):
        self.namespace = namespace
        return self

    @runtime_logging
    def set(self, key, file_path, del_local=True, set_all_area=False):
        if set_all_area:
            self.default_engine.set_namespace(namespace=self.namespace).set(key=key, file_path=file_path,
                                                                            del_local=del_local)
            for area in self.available_area:
                if area is not self.using_area:
                    self.getEngine(using_area=area).set_namespace(namespace=self.namespace).set(key=key,
                                                                                                file_path=file_path,
                                                                                                del_local=del_local)
        else:
            return self.default_engine.set_namespace(namespace=self.namespace).set(key=key, file_path=file_path,
                                                                                   del_local=del_local)

    @runtime_logging
    def get(self, key, local_file, default=None, auto_unzip=False, auto_delete_local_zip=True, auto_untar=False,
            auto_delete_local_tar=True):

        if not os.path.exists(os.path.dirname(local_file)):
            utils.make_dir(os.path.dirname(local_file))

        is_get = self.default_engine.set_namespace(namespace=self.namespace).get(key=key, local_file=local_file)
        if not is_get:
            if default:
                self.set(key=key, file_path=default, del_local=True)
                is_get = self.get(key=key, local_file=local_file)
            else:
                return False

        if is_get:
            if os.path.splitext(local_file)[-1] == '.zip' and auto_unzip:
                unzip_dir = os.path.dirname(local_file)
                if os.path.exists(local_file):
                    utils.unzip(file_name=local_file, dst_dir=unzip_dir)
                    if auto_delete_local_zip:
                        os.remove(local_file)
            elif os.path.splitext(local_file)[-1] == '.gz' and auto_untar:
                untar_dir = os.path.dirname(local_file)
                if os.path.exists(local_file):
                    utils.untar(file_name=local_file, dst_dir=untar_dir)
                    if auto_delete_local_tar:
                        os.remove(local_file)
            return True
        else:
            return False

    @runtime_logging
    def delete(self, key, delete_all_area=False):
        if delete_all_area:
            self.default_engine.set_namespace(namespace=self.namespace).delete(key=key)

            for area in self.available_area:
                if area is not self.using_area:
                    self.getEngine(using_area=area).set_namespace(namespace=self.namespace).delete(key=key)
            return True
        else:
            return self.default_engine.set_namespace(namespace=self.namespace).delete(key=key)

    @runtime_logging
    def list_cache_objects(self):
        return self.default_engine.set_namespace(namespace=self.namespace).list_cache_objects()

    @runtime_logging
    def list_dir(self, deep_num):
        return self.default_engine.set_namespace(namespace=self.namespace).list_dir(deep_num=deep_num)



import base64
import codecs
import glob
import hashlib
import json
import logging
import os
import re
import shutil
import subprocess
import sys
import tarfile
import time
import zipfile
from urllib import request, parse
from urllib.parse import quote, urlparse
from collections import Counter

import requests

from bbcloud_python_sdk.FileCache import FileCache

logging.basicConfig(level=logging.WARNING)


def download_file(url, save_path):
    """
    下载文件
    :param url: URL
    :param save_path: 保存文件名
    :return:
    """
    try:
        content = request.urlopen(url)
        basename = os.path.dirname(save_path)
        if not os.path.exists(basename):
            os.makedirs(basename)
        with open(save_path, 'wb') as f:
            f.write(content.read())
        return 'ok'
    except Exception as e:
        return e


def execute(command, cwd='/', login=False):
    """
    执行本地命令
    :param command: 命令
    :param cwd: 执行环境
    :return: 输出，错误信息(bytes类型)
    """
    if login:
        command = 'bash --login -c "%s" ' % command

    sub = subprocess.Popen(command, cwd=cwd, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE, shell=True)
    (stdout, stderr) = sub.communicate(timeout=3600)
    print(stdout.decode('utf-8', 'ignore'))
    if sub.returncode != 0:
        raise Exception(stderr.decode('utf-8', 'ignore'))


def delete_dir_then_move_dir(file_from, file_to):
    """
    删除文件夹并移动新文件夹替换之
    :param file_from:
    :param file_to:
    :return:
    """
    if os.path.exists(file_from):
        if os.path.exists(file_to):
            shutil.rmtree(file_to)
        shutil.move(file_from, file_to)


def delete_dir_then_copy_dir(file_from, file_to, symlinks=False, ignore=None):
    """
    删除文件夹并复制新文件夹替换之
    :param file_from:
    :param file_to:
    :param symlinks:
    :param ignore:
    :return:
    """
    if os.path.exists(file_from):
        if os.path.exists(file_to):
            shutil.rmtree(file_to)
        try:
            shutil.copytree(file_from, file_to, symlinks, ignore)
        except Exception as e:
            if isinstance(e, shutil.Error):
                pass


def make_dir(target_dir):
    """
    递归新建目录
    :param target_dir:
    :return:
    """
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)


def remake_dir(target_dir):
    """
    删除并重建目录
    :param target_dir:
    :return:
    """
    if os.path.exists(target_dir):
        shutil.rmtree(target_dir)
    os.makedirs(target_dir)


def pull_git_project(project_dir, git_branch):
    project_branch_dir = os.path.join(project_dir, git_branch)
    execute('git checkout .', project_branch_dir)
    execute('git clean -xfd ', project_branch_dir)
    execute('git pull origin %s' % (git_branch), project_branch_dir)


def clone_git_project(git_url, git_branch, clone_dir, origin_object_branch="master"):
    git_origin_object_branch_dir = os.path.join(clone_dir, origin_object_branch)
    clone_branch_dir = os.path.join(clone_dir, git_branch)
    if os.path.exists(clone_branch_dir) == False:
        make_dir(clone_branch_dir)

    if os.path.exists(git_origin_object_branch_dir):
        execute('git init', clone_branch_dir)
        execute('git remote add origin %s' % (git_url), clone_branch_dir)
        execute("echo ../../../%s/.git/objects/ > .git/objects/info/alternates" % (origin_object_branch),
                clone_branch_dir)
        execute("git pull origin %s:%s" % (git_branch, git_branch), clone_branch_dir)
        execute("git checkout %s" % (git_branch), clone_branch_dir)
    else:
        execute('git clone -b %s %s %s' % (origin_object_branch, git_url, origin_object_branch), clone_dir)
        clone_git_project(git_url, git_branch, clone_dir, origin_object_branch)


def sync_project_git_branch_dir(git_url, project_dir, branch_name):
    if os.path.exists(project_dir) == False:
        make_dir(project_dir)

    project_dir_branch_dir = os.path.join(project_dir, branch_name)
    if os.path.exists(os.path.join(project_dir_branch_dir, '.git')):
        pull_git_project(project_dir, branch_name)
    else:
        clone_git_project(git_url, branch_name, project_dir)


def get_gitlab_project_info_by_id(host, token, project_id):
    return json.loads(request.urlopen(
        '%s/api/v4/projects/%s?private_token=%s' % (host, project_id, token)).read())


def get_gitlab_project_info_by_url(web_url, token):
    parse = urlparse(web_url)
    host = parse.netloc
    path = parse.path[1:]
    url = 'http://%s/api/v4/projects/%s?private_token=%s' % (host, quote(path, safe=''), token)
    return json.loads(request.urlopen(url).read())


def download_gitlab_project(host, token, project_id, git_branch, dst_dir):
    download_file(
        "%s/api/v4/projects/%s/repository/archive.zip?sha=%s&private_token=%s" % (
            host,
            project_id, quote(git_branch), token), dst_dir)


def get_dir_files_size(src_path, file_size_dict={}, ignore_dir=''):
    files = os.listdir(src_path)
    for file in files:
        if os.path.splitext(file)[0].startswith('.') or \
                os.path.join(src_path, file) == os.path.join(src_path, ignore_dir):
            # 排除.开头的隐藏文件和目录
            continue
        file_path = os.path.join(src_path, file)
        if os.path.isdir(file_path):
            get_dir_files_size(file_path, file_size_dict)
        else:
            # 去除掉公共目录起始再存入json
            file_size_dict[file_path] = os.path.getsize(file_path)

    return file_size_dict


def create_json_file(dst_path, dist):
    with open(dst_path, 'w+', encoding='utf-8') as f:
        json.dump(dist, f, ensure_ascii=False, indent=4)


def get_md5_sum(file_path, block=4096):
    """
    获取MD5校验值
    :param file_path:
    :param block:
    :return:
    """
    with open(file_path, 'rb') as f:
        md5_obj = hashlib.md5()
        while 1:
            data = f.read(block)
            if not data:
                break
            md5_obj.update(data)

    return str(md5_obj.hexdigest()).lower()


def rm_dir_tree(path_list):
    """
    递归删除文件夹
    :param path_list: 文件路径列表
    :return:
    """
    for path in path_list:
        if os.path.exists(path) and os.path.isdir(path):
            shutil.rmtree(path)


def untar(file_name, dst_dir):
    tar = tarfile.open(file_name, "r:gz")
    file_names = tar.getnames()
    for file_name in file_names:
        tar.extract(file_name, dst_dir)
    tar.close()


def rm_file_by_path(path_list):
    """
    删除文件
    :param path_list: 文件路径列表
    :return:
    """
    for path in path_list:
        if os.path.exists(path):
            os.remove(path)


def copy_anything(src_path, dst_path, make_dirs=True):
    """
     自动识别是文件还是文件夹，并拷贝
    :param src_path:
    :param dst_path:
    :param make_dirs:
    :return:
    """
    if os.path.isfile(src_path):
        dst_folder_path = os.path.split(dst_path)[0]
        if not os.path.exists(dst_folder_path) and make_dirs:
            os.makedirs(dst_folder_path)
        shutil.copy(src_path, dst_path)
    else:
        shutil.copytree(src_path, dst_path)


def delete_then_replace_file(file_from, file_to):
    """
    删除文件并复制新文件替换之
    :param file_from:
    :param file_to:
    :return:
    """
    if os.path.exists(os.path.dirname(file_to)):
        if os.path.exists(file_to):
            os.remove(file_to)
    else:
        os.makedirs(os.path.dirname(file_to))
    shutil.copy(file_from, file_to)


def rm_dir(path):
    """
    递归删除文件夹
    :param path: 文件路径
    :return:
    """
    if os.path.exists(path) and os.path.isdir(path):
        shutil.rmtree(path)


def fory_tree(from_path, to_path, symlinks=False, is_pre_del=False):
    """
    递归复制文件夹
    :param from_path: 起始目录
    :param to_path: 目标目录
    :param symlinks: 是否保持符号链接
    :param is_pre_del: 是否执行前先删除目标目录
    :return:
    """
    if os.path.exists(from_path):
        if is_pre_del and os.path.exists(to_path):
            shutil.rmtree(to_path)

        try:
            shutil.copytree(from_path, to_path, symlinks)
        except Exception as e:
            if isinstance(e, shutil.Error):
                pass


def copy_dir_replace_file(src_path, dst_path, ignore=[], skip_hide_dir=True, skip_hide_file=True, replace=True):
    """
    递归复制目录，自动创建目录，遇到同名文件时可选是否替换
    :param src_path:
    :param dst_path:
    :param skip_hide_dir: 跳过隐藏目录
    :param skip_hide_file: 跳过隐藏文件
    :param replace: 替换同名文件
    :return:
    """
    if not os.path.exists(src_path):
        pass
    if not os.path.exists(dst_path):
        os.makedirs(dst_path)
    for root, dirs, files in os.walk(src_path):
        for file in files:
            if root.startswith('.') and skip_hide_dir:
                # 排除.开头的隐藏目录
                continue
            if os.path.splitext(file)[0].startswith('.') and skip_hide_file:
                # 排除.开头的隐藏文件
                continue
            in_ignore = False
            for item in ignore:
                if item in root:
                    in_ignore = True
            if in_ignore:
                continue

            dst_target_path = dst_path + root.split(src_path)[1]
            if not os.path.exists(dst_target_path):
                os.makedirs(dst_target_path)
            if replace:
                shutil.copy(os.path.join(root, file), dst_target_path)
            elif not os.path.isfile(os.path.join(dst_target_path, file)):
                shutil.copy(os.path.join(root, file), dst_target_path)


def aapt_verify_data_legality(dir_name):
    """
    检验文件的合法性
    :param dir_name:
    :return:
    """
    command = 'aapt package -A %s' % (dir_name)
    execute(command)


def fix_copy_tree(src, dest):
    try:
        shutil.copytree(src, dest)
    except Exception as e:
        if isinstance(e, shutil.Error):
            # 因为是远程共享目录，文件系统不一样，在复制的时候会因为元数据产生一些问题，但是文件似乎没有问题
            pass
        else:
            raise e


def replace_strings_in_file(old_new, file_path, encoding='utf-8'):
    """
    替换文件中的字符串
    :param old_new: 要替换的字符串,目标字符串
    :param file_path: 文件路径
    :param encoding: 编码格式
    :return:
    """
    with codecs.open(file_path, 'r', encoding) as f:
        strings = f.read()
        for old, new in old_new:
            strings = strings.replace(old, new)

    with open(file_path, 'wb+') as f:
        f.write(strings.encode('utf-8'))


def add_strings_in_file(new, file_path, encoding='utf-8'):
    """
    向文件添加新行写入字符串
    :param new: 新字符串
    :param file_path: 文件路径
    :param encoding: 编码格式
    :return:
    """
    with codecs.open(file_path, 'r', encoding) as f:
        m_str = f.read() + '\n' + new

    with open(file_path, 'w', encoding=encoding) as f:
        f.write(m_str)


def replace_string_in_file(old, new, file_path, encoding='utf-8'):
    """
    替换文件中的字符串
    :param old: 要替换的字符串
    :param new: 目标字符串
    :param file_path: 文件路径
    :param encoding: 编码格式
    :return:
    """
    with codecs.open(file_path, 'r', encoding) as f:
        strings = f.read().replace(old, new)
    with open(file_path, 'wb') as f:
        f.write(strings.encode('utf-8'))


def scramble(list_1, list_2):
    """
    判断list_1是否属于list2
    :return:
    """
    return len(Counter(list_1) - Counter(list_2)) == 0


def get_result_dict(log_obj):
    """
    获取结果返回值
    :return:
    """
    result_list = []
    for log_dict in log_obj.log_list:
        result_list.append('[code:%s]：%s' % (str(list(log_dict.keys())[0]), str(list(log_dict.values())[0])))
    log = '\n'.join(result_list)

    result_list.clear()
    for err_dict in log_obj.err_list:
        result_list.append('[code:%s]：%s' % (str(list(err_dict.keys())[0]), str(list(err_dict.values())[0])))
    err_log = '\n'.join(result_list)

    return {'log': log, 'err_log': err_log}


def remove_null_dir(target_path):
    """
    递归删除空目录
    :param target_path:
    :return:
    """
    if not os.path.exists(target_path):
        logging.info("[警告]尝试删除掉一个不存在的目录：%s" % target_path)
        return
    if os.path.isdir(target_path):
        for p in os.listdir(target_path):
            d = os.path.join(target_path, p)
            if os.path.isdir(d):
                remove_null_dir(d)
    if not os.listdir(target_path):
        os.rmdir(target_path)


def remove_hide_dir_file(path):
    """
    删除.开头的隐藏目录和文件
    :param path:
    :return:
    """
    for item in os.listdir(path):
        cur_path = os.path.join(path, item)
        if item.startswith('.'):
            if os.path.isdir(cur_path):
                shutil.rmtree(cur_path)
            else:
                try:
                    os.remove(cur_path)
                except OSError:
                    shutil.rmtree(cur_path)
        else:
            if os.path.isdir(cur_path):
                remove_hide_dir_file(cur_path)


def move_dir_replace_dir(src_path, dst_path, replace=True):
    """
    递归复制目录，自动创建目录，遇到同名文件时可选是否替换，完成之后再删除原目录（其实就相当于是剪切文件夹）
    :param src_path:
    :param dst_path:
    :param replace:
    :return:
    """
    copy_dir_replace_file(src_path, dst_path, replace=replace)
    if os.path.exists(src_path):
        shutil.rmtree(src_path)


def move_replace_dir(dir_from, dir_to):
    """
    移动并替换目录（dir_to只要指定到目标目录的父目录即可，否则会导致移动到dir_to里头的dir_to目录去）
    :param dir_from:
    :param dir_to:
    :return:
    """
    if os.path.exists(os.path.join(dir_to, os.path.basename(dir_from))):
        shutil.rmtree(os.path.join(dir_to, os.path.basename(dir_from)))

    if os.path.exists(dir_from):
        shutil.move(dir_from, dir_to)


def delete_file_rewrite_strings_to_file(file_path, content, encoding='utf-8'):
    """
    删除文件并重新写入字符串到文件
    :param file_path:
    :param content:
    :param encoding:
    :return:
    """
    if os.path.exists(file_path):
        os.remove(file_path)
    with open(file_path, 'wb+') as f:
        content_byte = content if isinstance(content, bytes) else content.encode(encoding)
        f.write(content_byte)


def replace_strings_in_string(old_new, string):
    """
    替换字符串中的字符串
    :param old_new: 要替换的字符串,目标字符串
    :param string: 字符串
    :return: 新字符串
    """
    for old, new in old_new:
        string = string.replace(old, new)
    return string


def post(url, data):
    """
    post请求
    :param url:
    :param data:
    :return:
    """
    req = request.Request(url)
    data = parse.urlencode(data)
    opener = request.build_opener(request.HTTPCookieProcessor())
    response = opener.open(req, data.encode('utf-8'))
    return response.read()


def remove_chinese_file(assets_dir):
    """
    移除中文文件
    :param path: 文件夹
    :return:
    """

    def is_chinese(char):
        """判断是否包含中文"""
        if not isinstance(char, str):
            char = char.decode('utf8')
        return re.search(r"[\u4e00-\u9fa5]+", char) is not None

    def is_space(char):
        """判断是否包含空格"""
        return re.search(r"\s", char) is not None

    def is_startswith(char):
        """检查字符串是否是以指定子字符串开头"""
        return char.startswith('%')

    res = []
    for root, dirs, files in os.walk(assets_dir):
        for f in files:
            if is_chinese(f) or is_space(f) or is_startswith(f):
                res.append(os.path.join(root, f))
        for d in dirs:
            if is_chinese(d) or is_space(d) or is_startswith(d):
                res.append(os.path.join(root, d))

    for r in res:
        if os.path.exists(r):
            if os.path.isdir(r):
                shutil.rmtree(r)
            else:
                os.remove(r)


def get_json_value(key, path, file_name='config.json'):
    """
    获得配置值
    :param key: 键
    :param path: 文件所在路径
    :param file_name: 文件名称
    :return:
    """
    c_file = os.path.join(path, file_name)

    if os.path.exists(c_file):
        with open(c_file) as f:
            c = json.load(f)
            if key in c:
                return c[key]
    return


def glud_snd(zh_path, zht_path):
    """
    传入两个i18n 替换zh的音频到zht
    :param zh_path: 简体中文目录
    :param zht_path: 繁体中文目录
    :return:
    """
    if not os.path.exists(zht_path):
        if os.path.exists(zh_path):
            shutil.copytree(zh_path, zht_path)
    elif os.path.exists(zh_path):
        m_list = os.walk(zh_path)
        for dirs, root, files in m_list:
            for m_file in files:
                tmp_dir = dirs.replace(zh_path, zht_path)
                if not os.path.exists(tmp_dir):
                    os.makedirs(tmp_dir)
                # 两边文件夹一致了，开始比较文件
                from_file = os.path.join(dirs, m_file)
                to_file = os.path.join(tmp_dir, m_file)
                if not os.path.exists(to_file):
                    shutil.copy(from_file, to_file)


def iterator_get_file_md5sum_to_json(path, rlt):
    """
    迭代获取目录下所有文件md5值
    :param path: 路径
    :param rlt: 结果
    :return:
    """
    paths = os.listdir(path)
    for _, item in enumerate(paths):
        sub_path = os.path.join(path, item)
        if os.path.isdir(sub_path):
            rlt[item] = {}
            iterator_get_file_md5sum_to_json(sub_path, rlt[item])
        else:
            rlt[item] = get_md5_sum(sub_path)


def iterator_get_file_md5sum_to_json_file(res_path, json_path):
    """
    迭代获取目录下所有文件md5值，并保存到json文件
    :param res_path: 路径
    :param json_path: 保存的json路径
    :return:
    """
    result = {}
    iterator_get_file_md5sum_to_json(res_path, result)
    with open(json_path, 'w') as f:
        json.dump(result, f)


def zip_files(zip_filename, file_list, arc_name=''):
    """
    压缩文件（列表）
    :param zip_filename: 压缩包名称
    :param file_list: 文件列表
    :param arc_name: 压缩包内的目录名称
    :return:
    """
    try:
        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED) as zf:
            for file in file_list:
                zf.write(file, os.path.join(arc_name, os.path.basename(file)))
    except Exception as e:
        raise Exception(e)


def zip_files_by_arc(zip_filename, file_dict):
    """
    压缩文件（集合）
    :param zip_filename: 压缩包名称
    :param file_dict: 文件列表
    :return:
    """
    try:
        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED) as zf:
            for file in file_dict:
                zf.write(file_dict[file], os.path.join(file, os.path.basename(file_dict[file])))
        return ''

    except Exception as e:
        return str(e)


def zip_dirs(zip_filename, file_dir, arc_name=''):
    """
    压缩目录
    :param zip_filename: 压缩包名称
    :param file_dir: 文件目录
    :param arc_name: 压缩包内的目录名称
    :return:
    """
    file_list = []

    try:
        for root, dirs, files in os.walk(file_dir):
            for dir_path in dirs:
                file_list.append(os.path.join(root, dir_path))
            for name in files:
                file_list.append(os.path.join(root, name))

        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED) as zf:
            for tar in file_list:
                zip_name = tar[len(file_dir):]
                if arc_name:
                    zip_name = os.path.join(arc_name, os.path.basename(zip_name))
                zf.write(tar, zip_name)
    except Exception as e:
        raise Exception(e)


def md5_convert(string):
    """
    计算字符串md5值
    :param string: 输入字符串
    :return: 字符串md5
    """
    m = hashlib.md5()
    m.update(string.encode())
    return m.hexdigest()


def base64_convert(string):
    """
    计算字符串base64值
    :param string: 输入字符串
    :return: 字符串base64
    """
    encodestr = base64.b64encode(string.encode('utf-8'))
    return encodestr


def retry(func):
    def inner(*args, **kwargs):
        count = 0
        while count < 3:
            try:
                func(*args, **kwargs)
                return
            except Exception as e:
                count += 1
                time.sleep(2)

    return inner


def get_middle_str(content, startStr, endStr):
    """
    获取文本字符串中间
    :param content:
    :param startStr:
    :param endStr:
    :return:
    """
    startIndex = content.index(startStr)
    if startIndex >= 0:
        startIndex += len(startStr)
    endIndex = content.index(endStr, startIndex)
    return content[startIndex:endIndex]


def get_right_string(html, start_str):
    """
    获取文本字符串右边
    :param html:
    :param start_str:
    :return:
    """
    start = html.find(start_str)
    if start >= 0:
        start += len(start_str)
        return html[start:-1].strip()
    return ""


def unzip(file_name, dst_dir=None):
    r = zipfile.is_zipfile(file_name)
    if r:
        fz = zipfile.ZipFile(file_name, 'r')
        for file in fz.namelist():
            fz.extract(file, dst_dir)
        fz.close()
    else:
        print('This is not zip')


def get_last_commit_sha_by_branch(git_host, git_token, git_project_id, git_branch):
    url = '%s/api/v4/projects/%s/repository/commits/%s' % (
        git_host, git_project_id, quote(git_branch, 'utf-8'))
    res = requests.request('GET', url,
                           params={
                               'private_token': git_token
                           },
                           stream=True)

    if res.status_code == 200:
        return json.loads(res.content)['id']
    else:
        raise Exception('commit sha get failure %s' % url)


def get_gitlab_last_pipeline_id_by_branch(git_host, git_token, git_project_id, git_branch):
    url = '%s/api/v4/projects/%s/pipelines' % (
        git_host, git_project_id)
    res = requests.request('GET', url,
                           params={
                               'private_token': git_token,
                               'status': 'success',
                               'ref': quote(git_branch, 'utf-8'),
                               'per_page': 1,
                               'order_by': 'id',
                               'sort': 'desc'
                           },
                           stream=True)

    if res.status_code == 200:
        return json.loads(res.content)[0]['id']
    else:
        raise Exception('pipeline id get failure %s %s' % (url, res.status_code))



import copy

from bbcloud_python_sdk.Api import BBCloud
from bbcloud_python_sdk.Api.Models.Query import Query


class Model(Query):

    def __init__(self, *args, **kwargs):
        super(Model, self).__init__()
        self.boot()
        self.loader(args[0] if args else kwargs)

    def __str__(self):
        model_str = '%s {\n' % self.__class__
        for k in self._attributes:
            model_str = model_str + "  '%s':'%s'\n" % (k, self._attributes[k])
        model_str = model_str + '}\n'
        return model_str

    def __repr__(self):
        return str('%s #%s' % (self.__class__, self.id))

    def __getattr__(self, attribute):
        try:
            return self.__dict__['_attributes'][attribute]
        except KeyError:
            raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, attribute))

    def __setattr__(self, key, value):
        if key in self.__dict__.get('_attributes', {}) and key != 'id':
            self.__dict__['_attributes'][key] = value
            self.__dict__['_changes'][key] = value
        else:
            super().__setattr__(key, value)

    def to_dict(self):
        return self._attributes

    def get_query(self):
        return {
            'q': self._query,
            'page': self._page,
            'page_size': self._page_size,
            'order_by': self._order_by
        }

    def get_modal_path(self):
        try:
            return getattr(self, '_modal_path')
        except AttributeError:
            model_name = self.__class__.__name__.lower()
            return '%s/%ss' % (model_name, self.__class__.__name__.lower())

    def reflection(self, data):
        if data:
            new_object = copy.deepcopy(self)
            new_object.boot()
            return new_object.loader(data)

    def boot(self):
        super(Model, self).boot()

        self._protocol = BBCloud.protocol
        # The model's attributes.
        self._attributes = {}
        # The model attribute's original state.
        self._original = {}
        # The changed model attributes.
        self._changes = {}

    def loader(self, data):
        self._attributes = data
        self._original = data
        return self

    def _get_key(self):
        return self.id if hasattr(self, 'id') else None

    def get(self):
        items = []
        data = BBCloud.protocol.set_request_query(
            query=self.get_query()
        ).set_request_path(
            request_path=self.get_modal_path()
        ).get()
        for item in data:
            items.append(self.reflection(data=item))
        return items

    def find(self, id):
        return self.where('id', id).first()

    def first(self):
        res = self.page(1).page_size(1).get()
        if res and len(res) >= 1:
            return res[0]
        else:
            return None

    def create(self, data=None):

        if not data:
            data = self._attributes

        if 'id' in data:
            data.pop('id')

        res = self._protocol.set_request_path(
            request_path=self.get_modal_path()
        ).create(data)

        return self.reflection(data=res)

    def save(self):
        key = self._get_key()
        if key:
            return self._protocol.set_request_path(
                request_path=self.get_modal_path()
            ).patch(id=key, data=self._changes)

    def delete(self):
        key = self._get_key()
        if key:
            return self._protocol.set_request_query(
                query=self._query
            ).set_request_path(
                request_path=self.get_modal_path()
            ).delete(id=key)
        else:
            items = self.get()
            res = []
            for item in items:
                res.append(item.delete())
            return all(res)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="bbcloud-python-sdk",
    version="0.0.17",
    author="BBCloud",
    author_email="zealiemai@gmail.com",
    description="bbcloud python sdk",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bbcloudGroup/bbcloud_python_sdk",
    packages=setuptools.find_packages(),
    install_requires=[
        'oss2==2.14.0',
        'synology-api==0.1.3.3',
        'esdk-obs-python==3.21.4',
        'requests==2.25.1'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
# python3 setup.py sdist bdist_wheel && python3 -m twine upload --repository pypi dist/*



class ParameterErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)


class MethodNotAllowedErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)


class ServiceErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)


class NetworkErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)



import calendar
import datetime
import logging
import os
import random
import re
import string
import time
import traceback
from collections import Counter
from distutils.sysconfig import PREFIX

import bbcloud_python_sdk
import oss2 as oss2


class OssFileCache():
    def __init__(self, access_key_id, access_key_secret, endpoint, bucket_name, cache_path_root,
                 namespace=''):
        self.cache_path_root = cache_path_root
        self.namespace = namespace
        auth = oss2.Auth(access_key_id=access_key_id,
                         access_key_secret=access_key_secret)
        self.bucket = oss2.Bucket(auth=auth, endpoint=endpoint,
                                  bucket_name=bucket_name)

    def random_str(self, num):
        salt = ''.join(random.sample(string.ascii_letters + string.digits, num))

        return salt

    def set_namespace(self, namespace):
        self.namespace = namespace
        logging.info('setting namespace %s' % namespace)
        return self

    def set(self, key, file_path, del_local=True):
        """
        缓存文件到OSS
        @param key: 缓存键
        @param file_path: 文件路径 url|本地目录|本地文件
        @param del_local: 缓存后是否删除本地文件
        """
        local_path = '/tmp/%s.%s' % (self.random_str(30), self.random_str(30))
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        if file_path:
            if isinstance(file_path, str) and re.match("http", file_path) and file_path is not None:
                bbcloud_python_sdk.download_file(url=file_path, save_path=local_path)
            elif isinstance(file_path, dict) or isinstance(file_path, list):
                bbcloud_python_sdk.create_json_file(dst_path=local_path, dist=file_path)
            elif os.path.isdir(file_path):
                bbcloud_python_sdk.zip_dirs(zip_filename=local_path, file_dir=file_path)
            elif os.path.isfile(file_path):
                local_path = file_path

            self.bucket.put_object_from_file(cache_path, local_path)

            if del_local:
                if os.path.exists(local_path):
                    os.remove(local_path)
            return True

    def get(self, key, local_file):
        """
        从OSS获取缓存文件
        @param key: 缓存键
        @param local_file: 文本保存路径
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        try:
            self.bucket.get_object_to_file(cache_path, local_file)
            return True
        except oss2.exceptions.NoSuchKey:
            if os.path.exists(local_file):
                os.remove(local_file)
            return False

    def delete(self, key):
        """
        删除OSS缓存文件
        @param key:
        """
        cache_path = "%s/%s/%s" % (self.cache_path_root, self.namespace, key)

        try:
            self.bucket.delete_object(cache_path)
            return True
        except oss2.exceptions.NoSuchKey:
            return False

    def list_cache_objects(self):
        """
        获取文件夹OSS缓存文件
        """
        prefix_path = "%s/%s" % (self.cache_path_root, self.namespace)
        list_dir = []
        marker = ''
        begin = False
        while (marker != '') or (not begin):
            list_objects = self.bucket.list_objects(prefix=prefix_path, max_keys=100, marker=marker)
            marker = list_objects.next_marker
            begin = True
            for i in list_objects.object_list:
                list_dir.append(i)
        return list_dir

    def list_dir(self, deep_num):
        prefix_path = "%s/%s" % (self.cache_path_root, self.namespace)
        all_list_dir = []
        one_list_dir = []
        marker = ''
        begin = False
        while (marker != '') or (not begin):
            list_objects = self.bucket.list_objects(prefix=prefix_path, max_keys=100, marker=marker)
            marker = list_objects.next_marker
            begin = True
            for i in list_objects.object_list:
                all_list_dir.append(i.key.rsplit("/", 1)[0])
        for data in all_list_dir:
            tmp_dir = data.rsplit("/", len(data.split('/')) - len(prefix_path.split('/')) - deep_num)[0]
            if tmp_dir not in one_list_dir:
                one_list_dir.append(tmp_dir)
        if self.cache_path_root in one_list_dir:
            one_list_dir.remove(self.cache_path_root)
        if prefix_path in one_list_dir:
            one_list_dir.remove(prefix_path)
        return one_list_dir



import json

import logging

import requests
from requests.structures import CaseInsensitiveDict

from bbcloud_python_sdk.Api.Exceptions import ParameterErrorException, MethodNotAllowedErrorException


class Http():
    def __init__(self, client):
        self.client = client

    def set_request_path(self, request_path):
        self.request_path = request_path
        return self

    def get_request_url(self):
        return '%s/%s' % (self.client.base_url, self.request_path)

    def set_request_query(self, query):
        self.query = query
        return self

    def set_params(self, params):
        self.params = params
        return self

    def get(self):
        res = self.do_request(
            req=Request(
                method='GET',
                url=self.get_request_url(),
                client=self.client,
                params=self.query
            )
        )
        if res:
            return res.get('data', [])

    def do_request(self, req, timeout=10):
        res = requests.request(req.method, req.url,
                               data=req.data,
                               params=req.params,
                               headers=req.headers,
                               stream=True,
                               timeout=timeout)
        return Response(res).read()

    def create(self, data):
        res = self.do_request(
            req=Request(
                method='POST',
                url=self.get_request_url(),
                client=self.client,
                data=json.dumps(data)
            )
        )
        return res

    def patch(self, id, data):
        res = self.do_request(
            req=Request(
                method='PATCH',
                url='%s/%s' % (self.get_request_url(), id),
                client=self.client,
                data=json.dumps(data)
            )
        )
        return res

    def delete(self, id):
        res = self.do_request(
            req=Request(
                method='DELETE',
                url='%s/%s' % (self.get_request_url(), id),
                client=self.client
            )
        )
        return res == 1


class Request(object):
    def __init__(self, method, url, client,
                 data=None,
                 params=None,
                 headers=None):
        self.method = method
        self.url = url
        self.data = data
        self.params = params or {}

        if not isinstance(headers, CaseInsensitiveDict):
            self.headers = CaseInsensitiveDict(headers)
        else:
            self.headers = headers

        self.headers['Content-Type'] = 'application/json;charset=UTF-8'

        self.headers['Authorization'] = 'Bearer %s' % (client.token)

        if client._auth_team:
            self.headers['Auth-team'] = client._auth_team

        if client._auth_application:
            self.headers['Auth-team'] = client._auth_application

        if client._auth_namespace:
            self.headers['Auth-team'] = client._auth_namespace

        logging.debug(self.method)
        logging.debug(self.headers)
        logging.debug(self.params)


class Response(object):
    def __init__(self, response):

        self.response = response
        self.status = response.status_code
        self.headers = response.headers
        if self.status == 422:
            raise ParameterErrorException(self.read())
        if self.status == 405:
            raise MethodNotAllowedErrorException(
                '%s method not allowed for %s' % (response.request.method, response.url)
            )

    def read(self):
        return json.loads(self.response.content)
